@@ -1,6 +1,6 @@
 /*
 
-$Header: /cvsroot/tsp/tsp/src/util/libbb/bb_core.c,v 1.24 2006/07/22 17:07:15 deweerdt Exp $
+$Header: /cvsroot/tsp/tsp/src/util/libbb/bb_core.c,v 1.27 2006/11/28 13:52:27 erk Exp $
 
 -----------------------------------------------------------------------
 
@@ -35,6 +35,7 @@ Purpose   : Blackboard Idiom implementation
 -----------------------------------------------------------------------
  */
 
+#ifndef __KERNEL__
 #include <string.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -51,6 +52,7 @@ Purpose   : Blackboard Idiom implementation
 #include <tsp_sys_headers.h>
 #include <tsp_simple_trace.h>
 
+#endif /* __KERNEL__ */
 
 #include "bb_alias.h"
 #include "bb_utils.h"
@@ -59,6 +61,7 @@ Purpose   : Blackboard Idiom implementation
 /**
  * Convert type to string for display use.
  */
+#ifndef __KERNEL__
 static const char* E_BB_2STRING[] = {"DiscoverType",
 				     "double", 
 				     "float", 
@@ -74,10 +77,16 @@ static const char* E_BB_2STRING[] = {"DiscoverType",
 				     "uchar",
 				     "UserType",
 				     "NotAType end"};
+#endif
 
 static const size_t E_BB_TYPE_SIZE[] = {0,
+			#ifndef __KERNEL__
 					sizeof(double), 
 					sizeof(float), 
+                        #else 
+                                        0,
+                                        0,
+			#endif
 					sizeof(int8_t),
 					sizeof(int16_t),
 					sizeof(int32_t),
@@ -91,18 +100,32 @@ static const size_t E_BB_TYPE_SIZE[] = {0,
 					0,
 					0};
 
+#ifdef __KERNEL__
+/* In case we're compiling kernel code, there's really no need
+   for the sysv code */
+static struct bb_operations sysv_bb_ops;
+extern struct bb_operations k_bb_ops;
+#else
 extern struct bb_operations sysv_bb_ops;
 extern struct bb_operations k_bb_ops;
+#endif
 
 /* Note: This is thigtly related to enum bb_type in bb_core.h,
    if you ever modify this, you'll probably need to modify
    enum bb_type */
 static struct bb_operations *ops[] = { 	&sysv_bb_ops,
-					NULL };
+					&k_bb_ops };
 
 static enum bb_type bb_type(const char *name)
 {
+#ifdef __KERNEL__
+	return BB_KERNEL;
+#else
+	if (!strncmp(name, "/dev/", 5))
+		return BB_KERNEL;
+	
 	return BB_SYSV;
+#endif /* __KERNEL__ */
 }
 
 size_t 
@@ -117,12 +140,14 @@ sizeof_bb_type(E_BB_TYPE_T bb_type) {
 E_BB_TYPE_T 
 bb_type_string2bb_type(const char* bb_type_string) {
   E_BB_TYPE_T retval = 0;
+#ifndef __KERNEL__
   if (!strncasecmp("double",bb_type_string,strlen("double"))) {
     retval = E_BB_DOUBLE;
   } else
   if (!strncasecmp("float",bb_type_string,strlen("float"))) {
     retval = E_BB_FLOAT;
   } else
+#endif
   if (!strncasecmp("int8",bb_type_string,strlen("int8"))) {
     retval = E_BB_INT8;
   } else
@@ -229,6 +254,7 @@ bb_data(volatile S_BB_T* bb) {
   return retval;
 } /* end of bb_data */
 
+#ifndef __KERNEL__
 double
 bb_double_of(void* value, E_BB_TYPE_T bbtype) {
 
@@ -280,6 +306,7 @@ bb_double_of(void* value, E_BB_TYPE_T bbtype) {
 
   return retval;
 } /* end of bb_double_of */
+#endif
 
 int32_t 
 bb_data_initialise(volatile S_BB_T* bb, S_BB_DATADESC_T* data_desc,void* default_value) {
@@ -302,12 +329,14 @@ bb_data_initialise(volatile S_BB_T* bb, S_BB_DATADESC_T* data_desc,void* default
   retval = BB_OK;
   for (i=0; i< data_desc->dimension; ++i) {
     switch (data_desc->type) {
+#ifndef __KERNEL__
     case E_BB_DOUBLE: 
       ((double*) data)[i] = default_value ? *((double *) default_value) : 0.0;
       break;
     case E_BB_FLOAT:
       ((float*) data)[i] = default_value ? *((float *) default_value) : 0.0;
       break;
+#endif
     case E_BB_INT8:
       ((int8_t*) data)[i] = default_value ? *((int8_t *) default_value) : 0;
       break;
@@ -357,6 +386,7 @@ bb_value_direct_rawwrite(void* data, S_BB_DATADESC_T data_desc, void* value) {
   return BB_OK;
 }
 
+#ifndef __KERNEL__
 int32_t
 bb_value_direct_write(void* data, S_BB_DATADESC_T data_desc, const char* value, int hexval) {
 
@@ -365,12 +395,14 @@ bb_value_direct_write(void* data, S_BB_DATADESC_T data_desc, const char* value,
   retval = BB_OK;
 
   switch (data_desc.type) {
+#ifndef __KERNEL__
   case E_BB_DOUBLE: 
     ((double *)data)[0] = atof(value);
     break;
   case E_BB_FLOAT:
     ((float *)data)[0] = atof(value);
     break;
+#endif
   case E_BB_INT8:
     ((int8_t*)data)[0] = strtol(value,(char **)NULL,hexval ? 16 : 10);
     break; 
@@ -415,9 +447,13 @@ bb_value_direct_write(void* data, S_BB_DATADESC_T data_desc, const char* value,
   }
   return retval;
 } /* bb_value_direct_write */
+#endif /* !__KERNEL__ */
 
 int32_t
-bb_value_write(volatile S_BB_T* bb, S_BB_DATADESC_T data_desc,const char* value, int32_t* idxstack, int32_t idxstack_len) {
+bb_value_write(volatile S_BB_T* bb, 
+	       S_BB_DATADESC_T data_desc, 
+	       const char* value, 
+	       int32_t* idxstack, int32_t idxstack_len) {
 
   char* data;
   int retval;
@@ -441,78 +477,95 @@ bb_value_write(volatile S_BB_T* bb, S_BB_DATADESC_T data_desc,const char* value,
   /* Get address of the data in BB */
   data = bb_item_offset(bb, &data_desc,idxstack,idxstack_len);
   /* Now write the value at obtained offset */
-
+#ifndef __KERNEL__
+  retval = bb_value_direct_write(data,data_desc,value,hexval);
+#endif
   return retval;
 } /* bb_value_write */
 
+#ifndef __KERNEL__
 
 int32_t
 bb_data_header_print(S_BB_DATADESC_T data_desc, FILE* pf, int32_t idx, int32_t aliastack) {
-	const char une_tab[] = "    ";
+  const char oneTab[] = "    "; 
+  char tabs[MAX_ALIAS_LEVEL*5]="";
+  
+  int i;
+  
+  for (i=0; i<(aliastack-1); i++)
+    {
+      strncat(tabs, oneTab, strlen(oneTab));
+    }
+  
+  fprintf(pf,"%s---------- < %s > ----------\n", tabs,data_desc.name);
+  fprintf(pf,"%s  alias-target = %d\n", tabs, data_desc.alias_target);
+  fprintf(pf,"%s  type         = %d  (%s)\n",tabs,data_desc.type,E_BB_2STRING[data_desc.type]);
+  fprintf(pf,"%s  dimension    = %d  \n",tabs,data_desc.dimension);
+  fprintf(pf,"%s  type_size    = %d  \n",tabs,data_desc.type_size);
+  fprintf(pf,"%s  data_offset  = %ld \n",tabs,data_desc.data_offset);
+  if ((idx>=0) && 
+      ((E_BB_UCHAR != data_desc.type) &&
+       (E_BB_CHAR  != data_desc.type)
+       )
+      ) {
+    fprintf(pf,"%s  value[%d]     = ",tabs,idx);
+  } else {
+    fprintf(pf,"%s  value        = ",tabs);
+  }
+  if ((data_desc.dimension > 1) &&
+      ((E_BB_UCHAR != data_desc.type) &&
+       (E_BB_CHAR  != data_desc.type)
+       )
+      ){
+    fprintf(pf," [ ");
+  } 
 
-	char decallage[MAX_ALIAS_LEVEL*5]="";
-	
-	int i;
-	
-	for (i=0; i<(aliastack-1); i++)
-	{
-		strncat(decallage, une_tab, strlen(une_tab));
-	}
-
-	fprintf(pf,"%s---------- < %s > ----------\n", decallage,data_desc.name);
-	fprintf(pf,"%s  alias-target = %d\n", decallage, data_desc.alias_target);
-	fprintf(pf,"%s  type         = %d  (%s)\n",decallage,data_desc.type,E_BB_2STRING[data_desc.type]);
-	fprintf(pf,"%s  dimension    = %d  \n",decallage,data_desc.dimension);
-	fprintf(pf,"%s  type_size    = %d  \n",decallage,data_desc.type_size);
-	fprintf(pf,"%s  data_offset  = %ld \n",decallage,data_desc.data_offset);
-	if (idx>=0) {
-	fprintf(pf,"%s  value[%d]    = ",decallage,idx);
-	} else {
-	fprintf(pf,"%s  value        = ",decallage);
-	}
-	if (data_desc.dimension > 1) {
-	fprintf(pf," [ ");
-	} 
-	if (idx>=0) {
-	fprintf(pf,"... ");
-	}
-	return 0;
+  if ((idx>=0) && 
+      ((E_BB_UCHAR != data_desc.type) &&
+       (E_BB_CHAR  != data_desc.type)
+       )
+      ) {
+    fprintf(pf,"... ");
+  }
+  return 0;
 } /* end of bb_data_header_print */
 
-
-
 int32_t
 bb_data_footer_print(S_BB_DATADESC_T data_desc, FILE* pf, int32_t idx, int32_t aliastack) {
-	const char une_tab[] = "    ";
+  const char oneTab[] = "    ";
+  char tabs[MAX_ALIAS_LEVEL*5]="";
+  
+  int i;
+  
+  for (i=0; i<(aliastack-1); i++) {
+    strncat(tabs, oneTab, strlen(oneTab));
+  }
+    
+  if ((idx>=0) &&
+      ((E_BB_UCHAR != data_desc.type) &&
+       (E_BB_CHAR  != data_desc.type)
+       )
+      ) {
+    fprintf(pf,"... ");
+  }  
 
-
-	char decallage[MAX_ALIAS_LEVEL*5]="";
-
-	int i;
-
-	for (i=0; i<(aliastack-1); i++)
-	{
-		strncat(decallage, une_tab, strlen(une_tab));
-	}
-
-
-	if (idx>=0) {
-		fprintf(pf,"... ");
-	}  
-	if (data_desc.dimension > 1) {
-		fprintf(pf,"]");
-	}  
-	fprintf(pf,"\n");    
-	fprintf(pf,"%s---------- ---------- ----------\n", decallage);
-
-	return 0;
+  if ((data_desc.dimension > 1) &&
+      ((E_BB_UCHAR != data_desc.type) &&
+       (E_BB_CHAR  != data_desc.type)
+       )
+      ){
+    fprintf(pf,"]");
+  }  
+  fprintf(pf,"\n");    
+  fprintf(pf,"%s---------- ---------- ----------\n", tabs);
+  
+  return 0;
 } /* end of bb_data_footer_print */
 
 int32_t 
-bb_value_print(volatile S_BB_T* bb, S_BB_DATADESC_T data_desc, FILE* pf, 
-               int32_t* idxstack, int32_t idxstack_len) {
-  
-  int32_t i,j,ibeg,iend;
+bb_string_value_print(volatile S_BB_T* bb, S_BB_DATADESC_T data_desc, FILE* pf, 
+		      int32_t* idxstack, int32_t idxstack_len) {
+  int32_t i,ibeg,iend;
   char* data;
   int32_t idx;
   assert(bb);
@@ -535,69 +588,146 @@ bb_value_print(volatile S_BB_T* bb, S_BB_DATADESC_T data_desc, FILE* pf,
     iend=data_desc.dimension;
   }
 
-  for (i=ibeg; i< iend; ++i) {
-    
-    switch (data_desc.type) {
-    case E_BB_DOUBLE: 
-      fprintf(pf,"%1.16f ",((double*) data)[i]);
-      break;
-    case E_BB_FLOAT:
-      fprintf(pf,"%f ",((float*) data)[i]);
-      break;
-    case E_BB_INT8:
-      fprintf(pf,"%d ",((int8_t*) data)[i]);
-      break;
-    case E_BB_INT16:
-      fprintf(pf,"%d ",((int16_t*) data)[i]);
-      break; 
-    case E_BB_INT32:
-      fprintf(pf,"%d ",((int32_t*) data)[i]);
-      break; 
-    case E_BB_INT64:
-      fprintf(pf,"%lld ",((int64_t*) data)[i]);
-      break;
-    case E_BB_UINT8:
-      fprintf(pf,"0x%x ",((uint8_t*) data)[i]);
-      break; 
-    case E_BB_UINT16:
-      fprintf(pf,"0x%x ",((uint16_t*) data)[i]);
-      break;
-    case E_BB_UINT32:
-      fprintf(pf,"0x%x ",((uint32_t*) data)[i]);
-      break;	
-    case E_BB_UINT64:
-      fprintf(pf,"0x%llx ",((uint64_t*) data)[i]);
-      break;	
-    case E_BB_CHAR:
-      fprintf(pf,"0x%02x<%c> ",((char*) data)[i],
-	      isprint((int)data[i]) ? ((char*) data)[i] : '?');
-      break;
-    case E_BB_USER:
-      for (j=0; j<data_desc.type_size; ++j) {
-	fprintf(pf,"0x%02x ",((uint8_t*) data)[i*data_desc.type_size+j]);
-        //bb_data_print(bb, *(&data_desc+1), pf);  
+  /* 
+   * If we have character this means we want
+   * a character-like or string-like display 
+   * HEX dump should be for UINT8/16/32/64
+   */
+  if ((E_BB_CHAR ==data_desc.type) ||
+      (E_BB_UCHAR==data_desc.type)) {
+    for (i=ibeg; i< iend; ++i) {
+      if ('\0'==((char*) data)[i]) {
+	break;
+      } else {
+	fprintf(pf,"%c",isprint((int)data[i]) ? ((char*) data)[i] : '?');
+      }
+    }
+  } else {
+    /* This is not a char type ? */
+    fprintf(pf,"bb_string_value_print: Not a char type");
+    return 1;
+  }
+  return 0;
+} /* end of bb_string_value_print */
+
+int32_t 
+bb_value_print(volatile S_BB_T* bb, S_BB_DATADESC_T data_desc, FILE* pf, 
+               int32_t* idxstack, int32_t idxstack_len) {
+  
+  int32_t i,j,ibeg,iend;
+  char* data;
+  int32_t idx;
+  int32_t charNullCount;
+  assert(bb);
+  /* We get BB data address  */
+  data = bb_item_offset(bb, &data_desc,idxstack,idxstack_len);
+  
+  /* check index stack to handle index */
+  if ((idxstack_len>0) && (data_desc.dimension > 1)){
+    idx = idxstack[idxstack_len-1];
+    if (idx>=0) {
+      ibeg=0;
+      iend=1;
+    } else {
+      ibeg=0;
+      iend=data_desc.dimension;
+    }
+  } else {
+    idx = 0;
+    ibeg=0;
+    iend=data_desc.dimension;
+  }
+
+  /* 
+   * If we have character this means we want
+   * a character-like or string-like display 
+   * HEX dump should be for UINT8/16/32/64
+   */
+  if ((E_BB_CHAR ==data_desc.type) ||
+      (E_BB_UCHAR==data_desc.type)) {
+    charNullCount = 0;
+    for (i=ibeg; i< iend; ++i) {
+      if ('\0'==((char*) data)[i]) {
+	charNullCount++;
+      } else {
+	if (charNullCount>0) {
+	  fprintf(pf,"(%d*'\\0')",charNullCount);
+	  charNullCount = 0;
+	}
+	if (isprint((int)data[i])) {
+	  fprintf(pf,"%c",((char*) data)[i]);
+	} else {
+	  fprintf(pf," 0x%02X ",((char*) data)[i]);
+	}
+      }
+    }
+    if (charNullCount>0) {
+      fprintf(pf," (%d*'\\0')",charNullCount);
+      charNullCount = 0;
+    }
+
+/*     fprintf(pf," (0x"); */
+/*     for (i=ibeg; i< iend; ++i) { */
+/*       fprintf(pf,"%02x",((char*) data)[i]); */
+/*     } */
+/*     fprintf(pf,")"); */
+  } else {
+
+    for (i=ibeg; i< iend; ++i) {    
+      switch (data_desc.type) {
+      case E_BB_DOUBLE: 
+	fprintf(pf,"%1.16f ",((double*) data)[i]);
+	break;
+      case E_BB_FLOAT:
+	fprintf(pf,"%f ",((float*) data)[i]);
+	break;
+      case E_BB_INT8:
+	fprintf(pf,"%d ",((int8_t*) data)[i]);
+	break;
+      case E_BB_INT16:
+	fprintf(pf,"%d ",((int16_t*) data)[i]);
+	break; 
+      case E_BB_INT32:
+	fprintf(pf,"%d ",((int32_t*) data)[i]);
+	break; 
+      case E_BB_INT64:
+	fprintf(pf,"%lld ",((int64_t*) data)[i]);
+	break;
+      case E_BB_UINT8:
+	fprintf(pf,"0x%x ",((uint8_t*) data)[i]);
+	break; 
+      case E_BB_UINT16:
+	fprintf(pf,"0x%x ",((uint16_t*) data)[i]);
+	break;
+      case E_BB_UINT32:
+	fprintf(pf,"0x%x ",((uint32_t*) data)[i]);
+	break;	
+      case E_BB_UINT64:
+	fprintf(pf,"0x%llx ",((uint64_t*) data)[i]);
+	break;	
+	/*     case E_BB_CHAR: */
+	/*       fprintf(pf,"0x%02x<%c> ",((char*) data)[i], */
+	/* 	      isprint((int)data[i]) ? ((char*) data)[i] : '?'); */
+	/*       break; */
+	/*     case E_BB_UCHAR: */
+	/*       fprintf(pf,"0x%02x<%c> ",((char*) data)[i], */
+	/* 	      isprint((int)data[i]) ? ((char*) data)[i] : '?'); */
+	/*       break;*/
+      case E_BB_USER:
+	for (j=0; j<data_desc.type_size; ++j) {
+	  fprintf(pf,"0x%02x ",((uint8_t*) data)[i*data_desc.type_size+j]);
+	  //bb_data_print(bb, *(&data_desc+1), pf);  
+	}
+	break; 
+      default:
+	fprintf(pf,"0x%x ",((char*) data)[i]);
+	break;
       }
-      break; 
-    default:
-      fprintf(pf,"0x%x ",((char*) data)[i]);
-      break;
     }
   } 
   return 0;
 } /* end of bb_value_print */
 
-
-
-/*int32_t 
-bb_data_print(volatile S_BB_T* bb, S_BB_DATADESC_T data_desc, FILE* pf) {
-	
-	bb_data_header_print(data_desc,pf,-1);
-	bb_value_print(bb,data_desc,pf,-1);
-	bb_data_footer_print(data_desc,pf,-1);
-	return BB_OK;
-
-}*/ /* end of bb_data_print */
-
 int32_t 
 bb_data_print(volatile S_BB_T* bb, S_BB_DATADESC_T data_desc, FILE* pf,
               int32_t* idxstack, int32_t idxstack_len) {
@@ -611,6 +741,7 @@ bb_data_print(volatile S_BB_T* bb, S_BB_DATADESC_T data_desc, FILE* pf,
    bb_data_footer_print(data_desc,pf,-1,aliasstack_size);
    return BB_OK;
 } /* end of bb_data_print */
+#endif /* !__KERNEL__ */
 
 int32_t 
 bb_create(S_BB_T** bb, 
@@ -636,6 +767,9 @@ bb_create(S_BB_T** bb,
 err:
   return retcode;
 } /* end of bb_create */
+#ifdef __KERNEL__
+EXPORT_SYMBOL_GPL(bb_create);
+#endif
 
 int32_t 
 bb_destroy(S_BB_T** bb) {
@@ -835,6 +969,7 @@ bb_item_offset(volatile S_BB_T *bb,
 } /* end of bb_item_offset */
 
 
+#ifndef __KERNEL__
 
 int32_t 
 bb_dump(volatile S_BB_T *bb, FILE* p_filedesc) {  
@@ -911,6 +1046,7 @@ bb_dump(volatile S_BB_T *bb, FILE* p_filedesc) {
   return retcode;
 } /* end of bb_dump */
 
+#endif /* ! __KERNEL__ */
 
 int32_t
 bb_get_nb_max_item(volatile S_BB_T *bb) {
@@ -1030,7 +1166,11 @@ bb_get_array_name(char * array_name,
   int32_t indexstack_curr;
   int j;
     
+#ifdef __KERNEL__
+  part_of_name = kmalloc(array_name_size_max, GFP_KERNEL);
+#else
   part_of_name = malloc(array_name_size_max);
+#endif /* __KERNEL__ */
 
   indexstack_curr = 0;
   for (j=aliasstack_size-1; j>=0; j--){
@@ -1063,7 +1203,11 @@ bb_get_array_name(char * array_name,
       strncat(array_name, part_of_name, array_name_size_max);
     }
   }
+#ifdef __KERNEL__
+  kfree (part_of_name);
+#else
   free (part_of_name);
+#endif
   return BB_OK;
 
 } /* end of get_array_name */
@@ -1074,6 +1218,21 @@ bb_msgq_isalive(S_BB_T *bb)
 	return ops[bb->type]->bb_msgq_isalive(bb);
 }
 
+#ifdef __KERNEL__
+int32_t 
+bb_logMsg(const BB_LOG_LEVEL_T level, const char* who, char* fmt, ...) 
+{
+	va_list args;	
+	char message[2048];
+
+	memset(message,0,2048);
+	va_start(args, fmt);
+	vsnprintf(message, 2048, fmt, args);
+	va_end(args);
+	printk("bb: %s : %s", who, message);
+	return 0;
+}
+#else
 int32_t 
 bb_logMsg(const BB_LOG_LEVEL_T level, const char* who, char* fmt, ...) {
   va_list args;
@@ -1117,5 +1276,6 @@ bb_logMsg(const BB_LOG_LEVEL_T level, const char* who, char* fmt, ...) {
   return 0;
 } /* end of bb_logMsg */
 
+#endif /* __KERNEL__ */
 
 
